@@ -19,7 +19,7 @@
 //!
 //! ```toml
 //! [dependencies]
-//! pem = "0.4"
+//! pem = "0.8"
 //! ```
 //!
 //! and this to your crate root:
@@ -84,7 +84,7 @@
 //! -----END CERTIFICATE-----
 //! ";
 //!
-//!  let pems = parse_many(SAMPLE);
+//!  let pems = parse_many(SAMPLE).unwrap();
 //!  assert_eq!(pems.len(), 2);
 //!  assert_eq!(pems[0].tag, "INTERMEDIATE CERT");
 //!  assert_eq!(pems[1].tag, "CERTIFICATE");
@@ -104,22 +104,15 @@
 )]
 
 mod errors;
+mod parser;
+use parser::{parse_captures, parse_captures_iter, Captures};
 
 pub use crate::errors::{PemError, Result};
-use lazy_static::lazy_static;
-use regex::bytes::{Captures, Regex};
 use std::str;
 
-const REGEX_STR: &str =
-    r"(?s)-----BEGIN (?P<begin>.*?)-----\s*(?P<data>.*?)-----END (?P<end>.*?)-----\s*";
-
 /// The line length for PEM encoding
 const LINE_WRAP: usize = 64;
 
-lazy_static! {
-    static ref ASCII_ARMOR: Regex = Regex::new(REGEX_STR).unwrap();
-}
-
 /// Enum describing line endings
 #[derive(Debug, Clone, Copy)]
 pub enum LineEnding {
@@ -137,7 +130,7 @@ pub struct EncodeConfig {
 }
 
 /// A representation of Pem-encoded data
-#[derive(PartialEq, Debug)]
+#[derive(PartialEq, Debug, Clone)]
 pub struct Pem {
     /// The tag extracted from the Pem-encoded data
     pub tag: String,
@@ -152,21 +145,13 @@ impl Pem {
         }
 
         // Verify that the begin section exists
-        let tag = as_utf8(
-            caps.name("begin")
-                .ok_or_else(|| PemError::MissingBeginTag)?
-                .as_bytes(),
-        )?;
+        let tag = as_utf8(caps.begin)?;
         if tag.is_empty() {
             return Err(PemError::MissingBeginTag);
         }
 
         // as well as the end section
-        let tag_end = as_utf8(
-            caps.name("end")
-                .ok_or_else(|| PemError::MissingEndTag)?
-                .as_bytes(),
-        )?;
+        let tag_end = as_utf8(caps.end)?;
         if tag_end.is_empty() {
             return Err(PemError::MissingEndTag);
         }
@@ -177,11 +162,7 @@ impl Pem {
         }
 
         // If they did, then we can grab the data section
-        let raw_data = as_utf8(
-            caps.name("data")
-                .ok_or_else(|| PemError::MissingData)?
-                .as_bytes(),
-        )?;
+        let raw_data = as_utf8(caps.data)?;
 
         // We need to get rid of newlines for base64::decode
         // As base64 requires an AsRef<[u8]>, this must involve a copy
@@ -242,8 +223,7 @@ impl Pem {
 ///  assert_eq!(pem.tag, "RSA PRIVATE KEY");
 /// ```
 pub fn parse<B: AsRef<[u8]>>(input: B) -> Result<Pem> {
-    ASCII_ARMOR
-        .captures(&input.as_ref())
+    parse_captures(&input.as_ref())
         .ok_or_else(|| PemError::MalformedFraming)
         .and_then(Pem::new_from_captures)
 }
@@ -278,7 +258,7 @@ pub fn parse<B: AsRef<[u8]>>(input: B) -> Result<Pem> {
 /// ";
 /// let SAMPLE_BYTES: Vec<u8> = SAMPLE.into();
 ///
-///  let pems = parse_many(SAMPLE_BYTES);
+///  let pems = parse_many(SAMPLE_BYTES).unwrap();
 ///  assert_eq!(pems.len(), 2);
 ///  assert_eq!(pems[0].tag, "INTERMEDIATE CERT");
 ///  assert_eq!(pems[1].tag, "CERTIFICATE");
@@ -312,16 +292,15 @@ pub fn parse<B: AsRef<[u8]>>(input: B) -> Result<Pem> {
 /// ";
 ///  let SAMPLE_STRING: Vec<u8> = SAMPLE.into();
 ///
-///  let pems = parse_many(SAMPLE_STRING);
+///  let pems = parse_many(SAMPLE_STRING).unwrap();
 ///  assert_eq!(pems.len(), 2);
 ///  assert_eq!(pems[0].tag, "INTERMEDIATE CERT");
 ///  assert_eq!(pems[1].tag, "CERTIFICATE");
 /// ```
-pub fn parse_many<B: AsRef<[u8]>>(input: B) -> Vec<Pem> {
+pub fn parse_many<B: AsRef<[u8]>>(input: B) -> Result<Vec<Pem>> {
     // Each time our regex matches a PEM section, we need to decode it.
-    ASCII_ARMOR
-        .captures_iter(&input.as_ref())
-        .filter_map(|caps| Pem::new_from_captures(caps).ok())
+    parse_captures_iter(&input.as_ref())
+        .map(|caps| Pem::new_from_captures(caps))
         .collect()
 }
 
@@ -562,12 +541,18 @@ RzHX0lkJl9Stshd/7Gbt65/QYq+v+xvAeT0CoyIg
 
     #[test]
     fn test_parse_many_works() {
-        let pems = parse_many(SAMPLE_CRLF);
+        let pems = parse_many(SAMPLE_CRLF).unwrap();
         assert_eq!(pems.len(), 2);
         assert_eq!(pems[0].tag, "RSA PRIVATE KEY");
         assert_eq!(pems[1].tag, "RSA PUBLIC KEY");
     }
 
+    #[test]
+    fn test_parse_many_errors_on_invalid_section() {
+        let input = SAMPLE_LF.to_owned() + "-----BEGIN -----\n-----END -----";
+        assert_eq!(parse_many(&input), Err(PemError::MissingBeginTag));
+    }
+
     #[test]
     fn test_encode_empty_contents() {
         let pem = Pem {
@@ -596,7 +581,7 @@ RzHX0lkJl9Stshd/7Gbt65/QYq+v+xvAeT0CoyIg
 
     #[test]
     fn test_encode_many() {
-        let pems = parse_many(SAMPLE_CRLF);
+        let pems = parse_many(SAMPLE_CRLF).unwrap();
         let encoded = encode_many(&pems);
 
         assert_eq!(SAMPLE_CRLF, encoded);
@@ -620,7 +605,7 @@ RzHX0lkJl9Stshd/7Gbt65/QYq+v+xvAeT0CoyIg
 
     #[test]
     fn test_encode_many_config() {
-        let pems = parse_many(SAMPLE_LF);
+        let pems = parse_many(SAMPLE_LF).unwrap();
         let config = EncodeConfig {
             line_ending: LineEnding::LF,
         };
